package core;

import gui.*;

import jxl.*;
import com.Ostermiller.util.*;

import java.util.*;
import java.text.*;
import java.io.*;

import javax.swing.*;

public class ConvertExcel2Xmdv {

	// row number and column number of excel file
	private int row, columns;

	// data set size and dimension number
	private int dataNumber, dimensionNumber;

	// path of excel file and okc file
	private String ExcelFilePath, OKCFilePath;

	// dimension names
	private ArrayList dimensionNames;

	// all data points,
	private ArrayList allData;

	// min and max value of each dimension
	private ArrayList MinMaxValue;

	// stack cardinality of each dimension
	private ArrayList dimStackCardinality;

	// the possible position of  error value in excel
	public int errRow, errCol;

	/**
	 * 
	 * @param excelFilePath
	 * @param okcFilePath
	 */

	public ConvertExcel2Xmdv(String excelFilePath, String okcFilePath) {

		this.ExcelFilePath = excelFilePath;
		this.OKCFilePath = okcFilePath;

		dimensionNames = new ArrayList();
		allData = new ArrayList();
		MinMaxValue = new ArrayList();
		dimStackCardinality = new ArrayList();

	}

	/**
	 * Set Stack Cardinality for each dimension
	 * 
	 * @param dimNum -
	 *            the dimension number
	 */
	private void SetDimStackCardinality(int dimNum) {
		dimStackCardinality.ensureCapacity(dimNum);
		for (int i = 0; i < dimNum; i++) {
			dimStackCardinality.add(Integer.valueOf(3));
		}
	}

	/**
	 * get an string type value from an excel sheet cell, indicating the row and
	 * column number
	 * 
	 * @param st -
	 *            excel sheet
	 * @param row -
	 *            row
	 * @param col -
	 *            column
	 * @return
	 */
	private String getFromSheet(Sheet st, int row, int col) {
		String str;
		Cell c = st.getCell(col, row);
		LabelCell labelc;
		str = c.getContents();

		if (c.getType() == CellType.LABEL) {
			labelc = (LabelCell) c;
			str = labelc.getString();
		}
		return str;
	}

	public int readCSV() {

		try {

			File f = new File(ExcelFilePath);

			if (!f.exists()) {
				return -1;
			}

			if (!ExcelFilePath.endsWith("csv")) {
				return -2;
			}

			InputStream in = new FileInputStream(f);

			LabeledCSVParser csvParser = new LabeledCSVParser(
					new ExcelCSVParser(in));
			int currLineNum = csvParser.getLastLineNumber();

			String[] currLine = null; //The data of current line.

			String[] Labels = csvParser.getLabels();
			
			if (Labels ==null){
				return -5;
			}
			columns = Labels.length;

			if (columns == 0) {
				return -5;
			}

			//set data point number and dimension number
			
			dimensionNumber = columns;

			//Set Stack Cardinality
			SetDimStackCardinality(columns);

			//set size for storing ArrayList

			dimensionNames.ensureCapacity(columns);

			String strc;
			int k;
			// get dimension names
			for (k = 0; k < columns; k++) {
				errRow = 1;
				errCol = k+1;
				strc = Labels[k];
				if(strc.equals("")){
					
					return -3;
				}
				dimensionNames.add(strc);
			}

			currLine = csvParser.getLine();

			if (currLine == null || currLine.length != columns) {
				return -3;
			} else {
				allData.ensureCapacity(1);
				row = 2;
			}

			ArrayList data1 = new ArrayList(columns);
			Double CurData1;

			// get the first data and set initial value of MinMaxValue
			int m;
			for (m = 0; m < columns; m++) {

				errRow = 2;
				errCol = m+1;
				strc = currLine[m];
				strc = strc.trim();
				if (strc.equals("")||strc==null) {
					
					return -3;
				}

				ArrayList CurMinMax = new ArrayList(2);

				CurData1 = Double.valueOf(strc);

				CurMinMax.add(CurData1);
				CurMinMax.add(CurData1);
				MinMaxValue.add(CurMinMax);

				data1.add(strc);

			}
			allData.add(data1);

			//read all the data values from the second data
			int j;

			currLine = csvParser.getLine();
			while (currLine != null) {
				if (currLine.length != columns) {
					return -3;
				}
				row++;
				ArrayList dataCur = new ArrayList(columns);
				ArrayList CurMinMax;
				double CurValue;
				for (j = 0; j < columns; j++) {

					strc = currLine[j];
					strc = strc.trim();
					errRow = row;
					errCol = j+1;
					if (strc.equals("")||strc==null) {
						return -3;
					}

					// compare and set min and max value
					CurMinMax = (ArrayList) MinMaxValue.get(j);
					CurValue = Double.parseDouble(strc);
					if (CurValue < (Double) (CurMinMax.get(0))) {
						CurMinMax.set(0, CurValue);
					} else if (CurValue > (Double) (CurMinMax.get(1))) {
						CurMinMax.set(1, CurValue);
					}
					dataCur.add(strc);
				}
				allData.add(dataCur);
				currLine = csvParser.getLine();
			}

			dataNumber = row - 1;
			csvParser.close();

		} catch (Exception e) {
			e.printStackTrace();
			return -3;
		}

		if (this.allData.isEmpty()) {
			return -4;
		}

		return 1;

	}

	/**
	 * read value from excel file and save dimension names, min and max valus in
	 * each dimension, and all data points value
	 * 
	 */

	public int readExcel() {

		try {

			File f = new File(ExcelFilePath);

			if (!f.exists()) {
				return -1;
			}

			if (!ExcelFilePath.endsWith("xls")) {
				return -2;
			}

			Workbook rwb = Workbook.getWorkbook(f);
			Sheet st = rwb.getSheet(0);

			// set row and column number of excel
			row = st.getRows();
			columns = st.getColumns();

			if (row == 1) {
				return -3;
			}

			if (row == 0 || columns == 0) {
				return -4;
			}

			// set data point number and dimension number
			dataNumber = row - 1;
			dimensionNumber = columns;

			// Set Stack Cardinality
			SetDimStackCardinality(columns);

			// set size for storing ArrayList
			allData.ensureCapacity(row - 1);
			dimensionNames.ensureCapacity(columns);

			Cell c;
			String strc;

			int k;
			// get dimension names
			for (k = 0; k < columns; k++) {
				strc = getFromSheet(st, 0, k);
				if (strc.equals("")||strc==null){
					errRow = 1;
					errCol = k+1;
					return -3;
				}
				dimensionNames.add(strc);
			}

			ArrayList data1 = new ArrayList(columns);
			Double CurData1;

			// get the first data and set initial value of MinMaxValue
			int m;
			for (m = 0; m < columns; m++) {
				
				errRow = 2;
				errCol = m+1;
				
				strc = getFromSheet(st, 1, m);

				strc = strc.trim();
				if (strc.equals("")||strc==null) {
					return -3;
				}

				ArrayList CurMinMax = new ArrayList(2);

				CurData1 = Double.valueOf(strc);

				CurMinMax.add(CurData1);
				CurMinMax.add(CurData1);
				MinMaxValue.add(CurMinMax);

				data1.add(strc);

			}
			allData.add(data1);

			// read all the data values from the second data
			int i, j;
			for (i = 2; i < row; i++) {
				
				System.out.println(i);
				ArrayList dataCur = new ArrayList(columns);
				ArrayList CurMinMax;
				double CurValue;
				for (j = 0; j < columns; j++) {

					errRow = i+1;
					errCol = j+1;
					
					System.out.println(j);
					strc = getFromSheet(st, i, j);

					strc = strc.trim();
					if (strc.equals("")||strc==null) {
						errRow = i+1;
						errCol = j+1;
						return -3;
					}

					// compare and set min and max value
					CurMinMax = (ArrayList) MinMaxValue.get(j);
					CurValue = Double.parseDouble(strc);
					if (CurValue < (Double) (CurMinMax.get(0))) {
						CurMinMax.set(0, CurValue);
					} else if (CurValue > (Double) (CurMinMax.get(1))) {
						CurMinMax.set(1, CurValue);
					}
					dataCur.add(strc);
				}
				allData.add(dataCur);

			}

			rwb.close();

		} catch (Exception e) {
			e.printStackTrace();
			return -3;
		}

		if (this.allData.isEmpty()) {
			return -4;
		}

		return 1;

	}

	/**
	 * write okc file
	 * 
	 */
	public int writeOKC() {

		// return if no data has been read;

		try {
			File okcf = new File(OKCFilePath);

			// create file folder if not exist
			if (!okcf.getParentFile().exists()) {
				new File(okcf.getParent()).mkdir();
			} else {
				// delete existing okc file
				if (okcf.exists()) {
					okcf.delete();
				}
			}
			okcf.createNewFile();

			
			FileWriter fw = new FileWriter(OKCFilePath, true);

			String str;

			// write dimension number
			str = Integer.toString(columns);
			fw.write(str + " ");

			// write data point number
			str = Integer.toString(row - 1);
			fw.write(str);

			fw.write("\r\n");

			// write dimension names
			for (int i = 0; i < dimensionNumber; i++) {
				fw.write((String) dimensionNames.get(i));
				fw.write("\r\n");
			}

			ArrayList al;

			// set the output value decimal format
			DecimalFormat df = new DecimalFormat("0.000000");

			// write min, max and cardinality number of each dimension
			for (int i = 0; i < dimensionNumber; i++) {
				al = (ArrayList) MinMaxValue.get(i);
				fw.write(df.format((Double) al.get(0))
								+ " "
								+ df.format((Double) al.get(1))
								+ " "
								+ Integer.toString((Integer) dimStackCardinality.get(i)));
				fw.write("\r\n");
			}

			int dn = dimensionNumber - 1;
			String s;
			double d;

			// write all data values
			for (int i = 0; i < dataNumber; i++) {
				al = (ArrayList) allData.get(i);
				for (int j = 0; j < dn; j++) {
					s = (String) al.get(j);
					d = Double.parseDouble(s);
					fw.write(df.format(d) + " ");
				}

				s = (String) al.get(dn);
				d = Double.parseDouble(s);

				fw.write(df.format(d) + "\r\n");

			}

			fw.close();

			

		} catch (Exception e) {
			e.printStackTrace();
			return -1;
		}
		return 1;
	}

	public static void main(String[] args) {
		try {
			new MainFrame();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
